(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@angular/forms'), require('rxjs'), require('rxjs/operators')) :
    typeof define === 'function' && define.amd ? define(['exports', '@angular/core', '@angular/forms', 'rxjs', 'rxjs/operators'], factory) :
    (factory((global.ngxerrors = {}),null,null,null,null));
}(this, (function (exports,core,forms,rxjs,operators) { 'use strict';

    var toArray = function (value) { return Array.isArray(value) ? value : [value]; };

    var NgxErrorsDirective = /** @class */ (function () {
        function NgxErrorsDirective(form) {
            this.form = form;
            this.ready = false;
        }
        Object.defineProperty(NgxErrorsDirective.prototype, "errors", {
            get: function () {
                if (!this.ready)
                    return;
                return this.control.errors;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgxErrorsDirective.prototype, "hasErrors", {
            get: function () {
                return !!this.errors;
            },
            enumerable: true,
            configurable: true
        });
        NgxErrorsDirective.prototype.hasError = function (name, conditions) {
            return this.checkPropState('invalid', name, conditions);
        };
        NgxErrorsDirective.prototype.isValid = function (name, conditions) {
            return this.checkPropState('valid', name, conditions);
        };
        NgxErrorsDirective.prototype.getError = function (name) {
            if (!this.ready)
                return;
            return this.control.getError(name);
        };
        NgxErrorsDirective.prototype.checkPropState = function (prop, name, conditions) {
            var _this = this;
            if (!this.ready)
                return;
            var controlPropsState = !conditions ||
                toArray(conditions).every(function (condition) { return _this.control[condition]; });
            if (name.charAt(0) === '*') {
                return this.control[prop] && controlPropsState;
            }
            return prop === 'valid'
                ? !this.control.hasError(name)
                : this.control.hasError(name) && controlPropsState;
        };
        NgxErrorsDirective.prototype.checkStatus = function () {
            var control = this.control;
            var errors = control.errors;
            this.ready = true;
            if (!errors)
                return;
            for (var errorName in errors) {
                this.subject$.next({ control: control, errorName: errorName });
            }
        };
        NgxErrorsDirective.prototype.ngOnInit = function () {
            this.subject$ = new rxjs.BehaviorSubject(null);
        };
        NgxErrorsDirective.prototype.ngOnChanges = function () {
            this.control = this.form.control.get(this.controlName);
        };
        NgxErrorsDirective.prototype.ngAfterViewInit = function () {
            var _this = this;
            setTimeout(function () {
                _this.checkStatus();
                _this.control.statusChanges.subscribe(_this.checkStatus.bind(_this), function () { return console.log('error'); }, function () { return console.log('completed'); });
            });
        };
        NgxErrorsDirective.prototype.ngOnDestroy = function () {
            this.subject$.complete();
        };
        NgxErrorsDirective.decorators = [
            { type: core.Directive, args: [{
                        selector: '[ngxErrors]',
                        exportAs: 'ngxErrors'
                    },] },
        ];
        /** @nocollapse */
        NgxErrorsDirective.ctorParameters = function () { return [
            { type: forms.FormGroupDirective, },
        ]; };
        NgxErrorsDirective.propDecorators = {
            "controlName": [{ type: core.Input, args: ['ngxErrors',] },],
        };
        return NgxErrorsDirective;
    }());

    var NgxErrorDirective = /** @class */ (function () {
        function NgxErrorDirective(ngxErrors) {
            this.ngxErrors = ngxErrors;
            this.hidden = true;
            this.rules = [];
            this.errorNames = [];
        }
        Object.defineProperty(NgxErrorDirective.prototype, "ngxError", {
            set: function (value) {
                this.errorNames = toArray(value);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgxErrorDirective.prototype, "when", {
            set: function (value) {
                this.rules = toArray(value);
            },
            enumerable: true,
            configurable: true
        });
        NgxErrorDirective.prototype.ngOnInit = function () {
            var _this = this;
            this._states$ = new rxjs.Subject();
            this.states$ = this._states$.asObservable().pipe(operators.distinctUntilChanged());
            var errors = this.ngxErrors.subject$.pipe(operators.filter(Boolean), operators.filter(function (obj) { return !!~_this.errorNames.indexOf(obj.errorName); }));
            var states = this.states$.pipe(operators.map(function (states) { return _this.rules.every(function (rule) { return !!~states.indexOf(rule); }); }));
            this.subscription = rxjs.combineLatest(states, errors).subscribe(function (_a) {
                var states = _a[0], errors = _a[1];
                _this.hidden = !(states && errors.control.hasError(errors.errorName));
            });
        };
        NgxErrorDirective.prototype.ngDoCheck = function () {
            var _this = this;
            this._states$.next(this.rules.filter(function (rule) { return _this.ngxErrors.control[rule]; }));
        };
        NgxErrorDirective.prototype.ngOnDestroy = function () {
            this._states$.complete();
            this.subscription.unsubscribe();
        };
        NgxErrorDirective.decorators = [
            { type: core.Directive, args: [{
                        selector: '[ngxError]'
                    },] },
        ];
        /** @nocollapse */
        NgxErrorDirective.ctorParameters = function () { return [
            { type: NgxErrorsDirective, decorators: [{ type: core.Inject, args: [core.forwardRef(function () { return NgxErrorsDirective; }),] },] },
        ]; };
        NgxErrorDirective.propDecorators = {
            "ngxError": [{ type: core.Input },],
            "when": [{ type: core.Input },],
            "hidden": [{ type: core.HostBinding, args: ['hidden',] },],
        };
        return NgxErrorDirective;
    }());

    var dependencies = [
        NgxErrorsDirective,
        NgxErrorDirective
    ];
    var NgxErrorsModule = /** @class */ (function () {
        function NgxErrorsModule() {
        }
        NgxErrorsModule.decorators = [
            { type: core.NgModule, args: [{
                        declarations: dependencies.slice(),
                        exports: dependencies.slice()
                    },] },
        ];
        return NgxErrorsModule;
    }());

    exports.NgxErrorDirective = NgxErrorDirective;
    exports.NgxErrorsDirective = NgxErrorsDirective;
    exports.NgxErrorsModule = NgxErrorsModule;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
